@@ -1,8 +1,10 @@
-from commodutil import transforms
-from commodutil import dates
 import re
+
 import pandas as pd
 
+from commodutil import dates
+from commodutil import transforms
+
 
 def curve_seasonal_zscore(hist, fwd):
     """
@@ -14,7 +16,11 @@ def curve_seasonal_zscore(hist, fwd):
 
     if isinstance(fwd, pd.Series):
         fwd = pd.DataFrame(fwd)
-    fwd['zscore'] = fwd.apply(lambda x: (d[x.name.month].loc['mean'] - x.iloc[0]) / d[x.name.month].loc['std'], 1)
+    fwd["zscore"] = fwd.apply(
+        lambda x: (d[x.name.month].loc["mean"] - x.iloc[0])
+        / d[x.name.month].loc["std"],
+        1,
+    )
     return fwd
 
 
@@ -24,12 +30,16 @@ def reindex_zscore(df, range=10):
     Essentially returns how far away the 'curve' is from historical trading range
     """
     df = transforms.reindex_year(df)
-    df = df.rename(columns={x: int(re.findall('\d\d\d\d', str(x))[0]) for x in df.columns})  # turn columns into years
-    d = df.loc[:, dates.curyear - range - 1:dates.curyear - 1]  # get subset of range years
-    d = d[:-10] # exclude last 10 rows to due to volatility close to expire
+    df = df.rename(
+        columns={x: int(re.findall("\d\d\d\d", str(x))[0]) for x in df.columns}
+    )  # turn columns into years
+    d = df.loc[
+        :, dates.curyear - range - 1 : dates.curyear - 1
+    ]  # get subset of range years
+    d = d[:-10]  # exclude last 10 rows to due to volatility close to expire
 
     dfs = []
-    for year in df.loc[:, dates.curyear:df.columns[-1]]:
+    for year in df.loc[:, dates.curyear : df.columns[-1]]:
         z = (d.mean(axis=1) - df.loc[:, year]) / d.std(axis=1)
         z.name = year
         dfs.append(z)
